// src/classes/Room.ts
var Room = class {
  namespace;
  users;
  state;
  server;
  /**
   * Constructs a new Room instance.
   * @param namespace The namespace identifier for the room.
   * @param server The instance of the CyberCafeServer that this room belongs to.
   */
  constructor(namespace, server) {
    this.state = {};
    this.server = server;
    this.namespace = this.server.io.of(namespace);
    this.users = /* @__PURE__ */ new Map();
    this.onCreate();
    this.namespace.on("connection", (socket) => {
      this.onMessage(socket.on);
      this.onJoin();
      socket.emit("stateUpdate", this.state);
    });
  }
  /**
   * Merges the provided state object with the current room state and broadcasts the updated state to all users.
   * @param update A partial state object with properties to merge into the current state.
   */
  updateState(update) {
    this.state = { ...this.state, ...update };
    this.namespace.emit("stateUpdate", this.state);
  }
  /**
   * Lifecycle hook that is called immediately after the room is created.
   * This method can be overridden by subclasses to perform custom initialization.
   */
  onCreate() {
  }
  /**
   * Lifecycle hook that is called when a user joins the room.
   * This method can be overridden by subclasses to handle user joining events.
   */
  onJoin() {
  }
  onMessage(on) {
  }
  /**
   * Adds a user to the room and associates them with a socket connection.
   * @param userId The unique identifier for the user.
   * @param socket The socket instance associated with the user.
   */
  addUser(userId, socket) {
    this.users.set(userId, socket);
    socket.join(this.namespace.name);
  }
  /**
   * Removes a user from the room based on their unique identifier.
   * @param userId The unique identifier for the user to remove.
   */
  removeUser(userId) {
    const user = this.users.get(userId);
    if (user) {
      user.leave(this.namespace.name);
      this.users.delete(userId);
    }
  }
  /**
   * Sends an event with associated data to all users in the room.
   * @param event The name of the event to broadcast.
   * @param data The data to be sent with the event.
   */
  broadcast(event, data) {
    this.namespace.emit(event, data);
  }
};

// src/classes/Server.ts
var CyberCafeServer = class {
  io;
  rooms;
  roomConstructors;
  /**
   * Initializes a new instance of the CyberCafeServer class.
   */
  constructor(io) {
    this.io = io;
    this.rooms = /* @__PURE__ */ new Map();
    this.roomConstructors = /* @__PURE__ */ new Map();
  }
  /**
   * Defines a new room type and its associated constructor.
   * @param registration The room registration object containing the room type and constructor.
   */
  defineRoomType(registration) {
    this.roomConstructors.set(
      registration.type,
      registration.constructor
    );
  }
  /**
   * Creates a new room of the specified type.
   * @param roomType The type of the room to create.
   * @param args The constructor arguments for the room.
   * @returns The newly created room instance.
   * @throws When the room type is unrecognized or the room ID is undefined.
   */
  createRoom(roomType, ...args) {
    const Constructor = this.roomConstructors.get(roomType);
    if (Constructor) {
      const room = new Constructor(...args, this);
      const roomId = args[0];
      if (roomId === void 0) {
        throw new Error("Room ID cannot be undefined");
      }
      this.rooms.set(roomId, room);
      return room;
    } else {
      throw new Error("Unrecognized room type");
    }
  }
  /**
   * Retrieves a room by its ID.
   * @param roomId The ID of the room to retrieve.
   * @returns The room instance if found, otherwise undefined.
   */
  getRoom(roomId) {
    return this.rooms.get(roomId);
  }
  /**
   * Removes a room from the server by its ID.
   * @param roomId The ID of the room to remove.
   */
  removeRoom(roomId) {
    this.rooms.delete(roomId);
  }
};
export {
  CyberCafeServer,
  Room
};
